<?php

declare(strict_types=1);

namespace Drupal\cache_control_override\EventSubscriber;

use Drupal\Core\Cache\CacheableResponseInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Cache Control Override.
 *
 * @internal
 *   There is no extensibility promise for this class. To override this
 *   functionality, you may subscribe to events at a higher priority, then
 *   set $event->stopPropagation(). The service may be decorated. Or you may
 *   remove, or replace this class entirely in service registration by
 *   implementing a ServiceProvider.
 */
final class CacheControlOverrideSubscriber implements EventSubscriberInterface {

  /**
   * Overrides cache control header if any of override methods are enabled.
   *
   * @param \Symfony\Component\HttpKernel\Event\ResponseEvent $event
   *   The event to process.
   */
  public function onRespond(ResponseEvent $event): void {
    if (FALSE === $event->isMainRequest()) {
      return;
    }

    $response = $event->getResponse();

    // If the current response isn't an implementation of the
    // CacheableResponseInterface, then there is nothing we can override.
    if (!$response instanceof CacheableResponseInterface) {
      return;
    }

    // If FinishResponseSubscriber didn't set the response as cacheable, then
    // don't override anything.
    if (FALSE === $response->headers->hasCacheControlDirective('max-age')) {
      return;
    }

    if (FALSE === $response->headers->hasCacheControlDirective('public')) {
      return;
    }

    $maxAge = $response->getCacheableMetadata()->getCacheMaxAge();

    // We treat permanent cache max-age as default therefore we don't override
    // the max-age.
    if ($maxAge !== CacheBackendInterface::CACHE_PERMANENT) {
      $response->headers->set('Cache-Control', 'public, max-age=' . $maxAge);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      KernelEvents::RESPONSE => [
        ['onRespond'],
      ],
    ];
  }

}
